package com.huiju.estateagents.redpack.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.redpack.entity.TaOrgOrder;
import com.huiju.estateagents.redpack.service.ITaOrgOrderService;

/**
 * <p>
    * 组织订单表  前端控制器
    * </p>
 *
 * @author fxf
 * @since 2020-06-03
 */
@RestController
@RequestMapping("/api")
public class TaOrgOrderController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaOrgOrderController.class);

    @Autowired
    public ITaOrgOrderService iTaOrgOrderService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/channel/taOrgOrder",method= RequestMethod.GET)
    public ResponseBean taOrgOrderList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @RequestParam( value = "orderType", required = false) String orderType,
                                       @RequestParam( value = "orgId", required = false) Integer orgId){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<TaOrgOrder> pg = new Page<>(pageNum, pageSize);
            IPage<TaOrgOrder> result = iTaOrgOrderService.listByCondition(pg, orderType, orgId);
            responseBean.addSuccess(result);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgOrderList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taOrgOrder 实体对象
     * @return
     */
    @RequestMapping(value="/taOrgOrder",method= RequestMethod.POST)
    public ResponseBean taOrgOrderAdd(@RequestBody TaOrgOrder taOrgOrder){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaOrgOrderService.save(taOrgOrder)){
                responseBean.addSuccess(taOrgOrder);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgOrderAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taOrgOrder/{id}", method= RequestMethod.DELETE)
    public ResponseBean taOrgOrderDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaOrgOrderService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgOrderDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taOrgOrder 实体对象
     * @return
     */
    @RequestMapping(value="/taOrgOrder/{id}",method= RequestMethod.PUT)
    public ResponseBean taOrgOrderUpdate(@PathVariable Integer id,
                                        @RequestBody TaOrgOrder taOrgOrder){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaOrgOrderService.updateById(taOrgOrder)){
                responseBean.addSuccess(taOrgOrder);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgOrderUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taOrgOrder/{id}",method= RequestMethod.GET)
    public ResponseBean taOrgOrderGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaOrgOrderService.getById(id));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgOrderDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
